print('Setting up WIFI...')
wifi.setmode(wifi.STATION)
station_cfg={}
station_cfg.ssid="WLAN NETZWERK"
station_cfg.pwd="WLAN KEY"
station_cfg.save=false
wifi.sta.config(station_cfg)
wifi.sta.connect()


m = mqtt.Client( "ledcontrol", 120, "MQTT USER", "MQTT PASSWORT" )
m:lwt( "/lwt", "offline", 0, 0 )
m:on( "message", function( client, topic, data )
	local status, mqtt_data = pcall( sjson.decode, data )
	if( status ) then
		process( mqtt_data )
	end
end)

pub = {}


tmr_mqtt = tmr.create()
tmr_wifi = tmr.create()
tmr_wifi:alarm( 1000, tmr.ALARM_AUTO, function( t )
	if wifi.sta.getip() == nil then
		print('Waiting for IP ...')
	else
--	if wifi.sta.setip( {
--			ip = "192.168.2.12",
--			netmask = "255.255.255.0",
--			gateway = "192.168.2.1"
--	} ) then
--		net.dns.setdnsserver( '9.9.9.10', 0 )

		print('IP is ' .. wifi.sta.getip())
		t:stop()

		m:connect( "MQTT IP ADDRESSE", 1883, false, function(client)
			print( "Connected to MQTT broker" )
			client:subscribe( "/ledcontrol", 0, function( client )
				print ( "Successfully subscribed to /ledcontrol" )

				table.insert( pub, '{ "state": "ready" }' )
				setup_inputs()

				dofile( "ledcontrol.lc" )
				LEDCtl:init()

				-- mqtt requires quite some resources, only send, when we're not busy.
				tmr_mqtt:alarm( 500, tmr.ALARM_AUTO, function( t )
					if( not running() ) then
						for key, value in next, pub do
							m:publish( "/ledcontrol", pub[ key ], 0, 1 )
						end
						pub = {}
					end
				end)

				tmr_mqtt:start()

				collectgarbage()

				-- lower CPU frequency to 80 MHz after setup
				-- runs more stable.
				node.setcpufreq( node.CPU80MHZ )
				node.egc.setmode(node.egc.ON_MEM_LIMIT, -6144)
				print( 'heap: ', node.heap() )
			end)

		end, function( client, reason )
			print("failed reason: " .. reason)
			tmr_mqtt:stop()
			t:start()
		end)
	end
end)

function process( data )
	if( data.request ~= nil ) then
		if( data.request.register ~= nil ) then
			for key, value in next, data.request.register do
				led_register( data.request.register[key].led, data.request.register[key].b )
			end
		elseif( data.request.fade ~= nil ) then
			if( data.request.fade == "start" ) then
				if( running() ) then
					table.insert( pub, '{ "reply": "Running new scene. Must stop first!" }' )
				else
					tmr_fade = tmr.create()
					tmr_fade:alarm( 100, tmr.ALARM_SINGLE, function( t )
						fade_start()
					end)
					table.insert( pub, '{ "reply": "Fading started." }' )
				end
				tmr_running = tmr.create()
				tmr_running:alarm( 1000, tmr.ALARM_AUTO, function( t )
					if( not running() ) then
						t:unregister()
						table.insert( pub, '{ "reply": "Fading finished." }' )
					end
				end)
			elseif( data.request.fade == "stop" ) then
				fade_stop()
				table.insert( pub, '{ "reply": "Fading stopped." }' )
			end
		elseif( data.request.setspeed ~= nil ) then
			--local status, retval = pcall(  )
			if( setSpeed( data.request.setspeed ) ) then
				table.insert( pub, '{ "reply": "Set speed." }' )
			else
				table.insert( pub, '{ "reply": "Speed must be >= 5 or <= 10000" }' )
			end
		elseif( data.request.setgamma ~= nil ) then
			if( LEDCtl:set_gamma( data.request.setgamma ) ) then
				table.insert( pub, '{ "reply": "Set gamma." }' )
			end
		elseif( data.request.setinit ~= nil ) then
			for key, value in next, data.request.setinit do
				setInit( data.request.setinit[ key ].led, data.request.setinit[ key ].b )
			end
		end
	end
end

function setup_inputs()
	-- New Board uses GPIO: GPIO 14, 12, 13, 9, 10, 2
	-- Rev 1 using GPIO 4, 5,  9, 10, 12, 13
	-- NodeMCU:         2, 1, 11, 12,  6,  7
	local inputs = { 2, 1, 11, 12, 6, 7 }
	for pin = 1, 6, 1 do
		print( "Pin "..pin.." GPIO "..inputs[ pin ] )
		gpio.mode( inputs[ pin ], gpio.INT )
		-- gpio.mode( inputs[ pin ], gpio.INT, gpio.PULLUP )
		gpio.trig( inputs[ pin ], "both", function( level, when, count )
			local action = ( level == gpio.HIGH ) and "released" or "pressed"
			--print( '{ "switch-event": { "switch": '..pin..', "action": "'..action..'" } }' )
			table.insert( pub, '{ "switch-event": { "switch": '..pin..', "action": "'..action..'" } }' )
		end)
	end
end
